#!/usr/bin/perl -w

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

# Here is a program that run a MonteCarlo Simulation to find out the
# probability of having # losses in a row.

# Since the code does one million trials of 1000 trades per trial, the
# expected value of the error is roughly 1/sqrt(1000000) = 1/1000 = 0.1 %

$probability_of_a_loss_on_any_one_trade = 0.60;
$nb = 45;

*OUT = *STDOUT;

for($i = 0; $i <= $nb; $i++) {
    $results[$i] = 0;
    $number_of_occurences_in_all_trials[$i] = 0;
}

print "............ Number of ..... Number of trials\n";
print "............ occurences .... in which this was\n";
print "# Losses ... in 1000000 .... the MAX # losses\n";
print "in a row ... trials ........ in a row\n";
print "==============================================\n";

for($i = 0; $i < 1000000; $i++) {

    $losses_in_a_row_in_this_trial = 0;
    $max_losses_in_a_row_in_this_trial = 0;

    for($j = 0; $j < 1000; $j++) {

	$random_number = rand();
	if ($random_number <= $probability_of_a_loss_on_any_one_trade) {
	    $losses_in_a_row_in_this_trial += 1;	
	    if ($losses_in_a_row_in_this_trial > $max_losses_in_a_row_in_this_trial) {
		$max_losses_in_a_row_in_this_trial = $losses_in_a_row_in_this_trial;
	    }
	} else {
	    $losses_in_a_row_in_this_trial = 0;
	}
    }
    # record max number of losses in a row recorded in this trial
    $results[$max_losses_in_a_row_in_this_trial] += 1;
}

for($i = 0; $i < $nb; $i++) {
    $number_of_occurences_in_all_trials[$nb - $i - 1] = $number_of_occurences_in_all_trials[$nb - $i] + $results[$nb - $i - 1];
}

for($i = 0; $i < $nb; $i++) {
    my $format = '@>>>>>    |    @>>>>>>>    |    @>>>>>>>';
    my $text = swrite($format, $i, $number_of_occurences_in_all_trials[$i], $results[$i]);
    print OUT "$text\n";
}

sub swrite {
    my $format = shift;
    $^A = "";
    formline($format,@_);
    return $^A;
}
