package GT::CloseStrategy::Generic;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use Carp::Datum;
use GT::Prices;
use GT::CloseStrategy;
use GT::Eval;
use GT::Tools qw(:generic);

@ISA = qw(GT::CloseStrategy);
@NAMES = ("Generic[#*]");

=head1 CloseStrategy::Generic

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => $args };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    my @sig1 = shift_args_until("Signals:", @args);
    
    $self->{'buy'} = create_standard_object(@sig1);
    $self->{'buy_number'} = extract_object_number(@sig1);
    $self->add_signal_dependency($self->{'buy'}, 1);
    if (scalar @args)
    {
	my @sig2 = shift_args_until("Signals:", @args);
	$self->{'sell'} = create_standard_object(@sig2);
	$self->{'sell_number'} = extract_object_number(@sig2);
	$self->add_signal_dependency($self->{'sell'}, 1);
    } else {
	$self->{'sell'} = $self->{'buy'};
	$self->{'sell_number'} = $self->{'buy_number'};
    }
}

sub precalculate_interval {
    my ($self, $calc, $first, $last) = @_;

    $self->{'buy'}->detect_interval($calc, $first, $last);
    if ($self->{'sell'} != $self->{'buy'})
    {
	$self->{'sell'}->detect_interval($calc, $first, $last);
    }
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return if (! $self->check_dependencies($calc, $i));

    if ($calc->signals->get($self->{'buy'}->get_name(
						$self->{'buy_number'}), $i))
    {
        my $order = $pf_manager->sell_market_price($calc, 
                                                   $sys_manager->get_name);
        $pf_manager->submit_order_in_position($position, $order, $i, $calc);
    }
    
    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    
    return if (! $self->check_dependencies($calc, $i));

    if
	($calc->signals->get($self->{'sell'}->get_name(
						$self->{'sell_number'}), $i))
    {
        my $order = $pf_manager->buy_market_price($calc, 
                                                  $sys_manager->get_name);
        $pf_manager->submit_order_in_position($position, $order, $i, $calc);
    }
   
    return;
}

1;
