package GT::CloseStrategy::LimitPeriodInTheMarket;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::CloseStrategy;
use GT::Prices;
use Carp::Datum;

@ISA = qw(GT::CloseStrategy);
@NAMES = ("LimitPeriodInTheMarket[#1]");

=head1 GT::CloseStrategy::LimitPeriodInTheMarket

This strategy closes the position once the allowed period of holding a
security, and to stay in the market, is exceded.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 30 ] };

    $args->[0] = 30 if (! defined($args->[0]));

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub long_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub short_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $initial_period = $calc->prices->date($position->{'open_date'});
    my $period_in_the_market = $self->{'args'}[0];
    
    if (($i + 1) eq ($initial_period + $period_in_the_market)) {
	my $order = $pf_manager->sell_market_price($calc, $sys_manager->get_name);
	$pf_manager->submit_order_in_position($position, $order, $i, $calc);
    }
    
    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $initial_period = $calc->prices->date($position->{'open_date'});
    my $period_in_the_market = $self->{'args'}[0];
    
    if (($i + 1) eq ($initial_period + $period_in_the_market)) {
	my $order = $pf_manager->buy_market_price($calc, $sys_manager->get_name);
	$pf_manager->submit_order_in_position($position, $order, $i, $calc);
    }
   
    return;
}

