package GT::CloseStrategy::Stop::BasedOnIndicators;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::CloseStrategy;
use Carp::Datum;
use GT::Eval;
use GT::Prices;
use GT::Tools qw(:generic);

@ISA = qw(GT::CloseStrategy);
@NAMES = ("BasedOnIndicators[#*]");

=head1 GT::CloseStrategy::Stop::BasedOnIndicators

=head2 Overview

This strategy end up a position once prices have crossed the trailing stop
determined by indicators.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [] };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    my @function1 = shift_args_until("Indicators:", @args);
    $self->{'f1'} = create_standard_object(@function1);
    $self->{'f1_number'} = extract_object_number(@function1);
    $self->add_indicator_dependency($self->{'f1'}, 1);

    if (scalar @args) {
        my @function2 = shift_args_until("Indicators:", @args);
        $self->{'f2'} = create_standard_object(@function2);
        $self->{'f2_number'} = extract_object_number(@function2);
        $self->add_indicator_dependency($self->{'f2'}, 1);
    } else {
        $self->{'f2'} = $self->{'f1'};
        $self->{'f2_number'} = $self->{'f1_number'};
    }
}

sub get_indicative_long_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $name = $self->{'f1'}->get_name($self->{'f1_number'});

    return 0 if (! $self->check_dependencies($calc, $i));
    my $price = $calc->indicators->get($name, $i);

    if ($price < $calc->prices->at($i)->[$LAST]) {
        return $price;
    } else {
	return 0;
    }
}

sub get_indicative_short_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $name = $self->{'f2'}->get_name($self->{'f2_number'});

    return 0 if (! $self->check_dependencies($calc, $i));
    my $price = $calc->indicators->get($name, $i);

    if ($price > $calc->prices->at($i)->[$LAST]) {
	return $price;
    } else {
	return 0;
    }
}

sub long_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub short_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $name = $self->{'f1'}->get_name($self->{'f1_number'});

    return if (! $self->check_dependencies($calc, $i));
    my $price = $calc->indicators->get($name, $i);

    if ($price < $calc->prices->at($i)->[$LAST]) {
	$position->set_stop($calc->indicators->get($name, $i));
    }
    
    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $name = $self->{'f2'}->get_name($self->{'f2_number'});

    return if (! $self->check_dependencies($calc, $i));
    my $price = $calc->indicators->get($name, $i);

    if ($price > $calc->prices->at($i)->[$LAST]) {
	$position->set_stop($calc->indicators->get($name, $i));
    }

    return;
}

1;
