package GT::CloseStrategy::Stop::Fixed;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::CloseStrategy;
use GT::Prices;
use Carp::Datum;

@ISA = qw(GT::CloseStrategy);
@NAMES = ("StopFixed[#1]");

=head1 GT::CloseStrategy::Stop::Fixed

This strategy closes the position once the prices have crossed a limit
called stop. This stop is defined as a percentage from the initial price.
The limit is parameterized. By default it's 4%.

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 4 ] };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;
    $self->{'long_factor'} = 1 - $self->{'args'}[0] / 100;
    $self->{'short_factor'} = 1 + $self->{'args'}[0] / 100;
}

sub get_indicative_long_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $stop = 0;
    if ($order->price)
    {
	$stop = $order->price * $self->{'long_factor'};
    } else {
	$stop = $calc->prices->at($i)->[$LAST] * $self->{'long_factor'};
    }
    return $stop;
}

sub get_indicative_short_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $stop = 0;
    if ($order->price)
    {
	$stop = $order->price * $self->{'short_factor'};
    } else {
	$stop = $calc->prices->at($i)->[$LAST] * $self->{'short_factor'};
    }
    return $stop;
}

sub long_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    $position->set_stop($position->open_price * $self->{'long_factor'});
    
    return;
}

sub short_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    $position->set_stop($position->open_price * $self->{'short_factor'});

    return;
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $ps_manager, $sys_manager) = @_;
    
    return;
}

