package GT::Indicators::Generic::MaxInPeriod;

# Copyright 2000-2002 Raphal Hertzog
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

# Standards-Version: 1.0

use strict;
use vars qw(@ISA @NAMES @DEFAULT_ARGS);

use GT::Indicators;
use GT::Prices;
use GT::Tools qw(:math);

@ISA = qw(GT::Indicators);
@NAMES = ("MaxInPeriod[#*]");
@DEFAULT_ARGS = (2, "{I:Prices CLOSE}");

=head1 NAME

GT::Indicators::Generic::MaxInPeriod - Calculate a maximum

=head1 DESCRIPTION

This indicator calculates the maximum of any serie of data in the
last XX days or since a given date.

=head1 PARAMETERS

=over

=item Number of days / date

You can specify either a number or a date. In the first case the maximum will
be calculated with the last <number> days. In the second case it will be the maximum
since the given date.

=item Data

This is the data to use as input. If you don't specify anything, the
closing price will be used by default.

=back

Example of accepted argument list :

=over

=item 10 {I:RSI}

=item 2002-01-03

=item 5

=item 2002-04-05 {I:Prices LOW}

=back

=cut
sub initialize {
    my ($self) = @_;
}

sub calculate {
    my ($self, $calc, $i) = @_;
    my $name = $self->get_name;
    my $getvalue = $self->{'func'};
    
    return if ($calc->indicators->is_available($name, $i));

    my $res = undef;
    my $arg = $self->{'args'}->get_arg_values($calc, $i, 1);

    if ($arg =~ /^\d+$/) {
	$res = &$getvalue($calc, $i);
	for (my $n = 1; $n < $arg; $n++) {
	    my $val = &$getvalue($calc, $i - $n);
	    if (defined($val) & defined($res)) {
		$res = max($res, $val);
	    }
	}
    } elsif ($arg =~/^\d{4}-\d\d-\d\d$/) {
	my $n = undef;
	if ($calc->prices->has_date($arg)) {
	    $n = $calc->prices->date($arg);
	} else {
	    $n = $calc->prices->date($calc->prices->find_nearest_date($arg));
	}
	if ($i >= $n) {
	    $res = &$getvalue($calc, $n++);
	    for(;$n <= $i; $n++) {
		my $val = &$getvalue($calc, $n);
		if (defined($val)) {
		    $res = max($res, $val);
		}
	    }
	}
    }
    
    if (defined($res)) {
	$calc->indicators->set($name, $i, $res);
    }
}
