package GT::Indicators::Generic::PeriodAgo;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Indicators;
use GT::Prices;
use GT::Tools qw(:generic);
use GT::Eval;

@ISA = qw(GT::Indicators);
@NAMES = ("PeriodAgo[#*]");

=head2 GT::Indicators::Generic::PeriodAgo

This function returns N-Period Ago value of the indicator given on
arguments. Without indicator, the close price is used.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my ($args, $key, $func) = @_;
    my $self = { 'args' => defined($args) ? $args : [ 1 ] };

    $args->[0] = 1 if (! defined($args->[0]));
    
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, $key);
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    if (scalar @args)
    {
	my @function1 = shift_args_until("Indicators", @args);
	$function1[0] = "Indicators:" . $function1[0];
	$self->{'object'} = create_standard_object(@function1);
	$self->{'f1_number'} = extract_object_number(@function1);
	$self->{'f1'} = sub { 
	    $_[0]->indicators->get(
		$self->{'object'}->get_name($self->{'f1_number'}), 
		$_[1])
	};
	$self->add_indicator_dependency($self->{'object'}, $self->{'args'}[0] + 1);
    } else {
	$self->{'f1'} = $GET_CLOSE;
	$self->add_prices_dependency($self->{'args'}[0] + 1);
    }
}

sub calculate {
    my ($self, $calc, $i) = @_;
    my $getvalue = $self->{'f1'};
    my $period = $self->{'args'}[0];
    my $name = $self->get_name;
    
    return if ($calc->indicators->is_available($name, $i));
    return if (! $self->check_dependencies($calc, $i));
    
    my $value = &$getvalue($calc, $i - $period);
    $calc->indicators->set($name, $i, $value);
}
