package GT::Indicators::Generic::Speed;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Indicators;
use GT::Prices;
use GT::Tools qw(:generic);
use GT::Eval;

@ISA = qw(GT::Indicators);
@NAMES = ("Speed[#*]","PercentSpeed[#*]");

=head2 GT::Indicators::Generic::Speed

This function returns today's value minus yesterday's value of the indicator
given on arguments. Without indicator, the close price is used.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my ($args, $key, $func) = @_;
    my $self = { 'args' => defined($args) ? $args : [] };
    
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, $key);
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    if (scalar @args)
    {
	my @function1 = shift_args_until("Indicators", @args);
	$function1[0] = "Indicators:" . $function1[0];
	$self->{'object'} = create_standard_object(@function1);
	$self->{'f1_number'} = extract_object_number(@function1);
	$self->{'f1'} = sub { 
	    $_[0]->indicators->get(
		$self->{'object'}->get_name($self->{'f1_number'}), 
		$_[1])
	};
	$self->add_indicator_dependency($self->{'object'}, 2);
    } else {
	$self->{'f1'} = $GET_CLOSE;
	$self->add_prices_dependency(2);
    }
}

sub calculate {
    my ($self, $calc, $i) = @_;
    my $getvalue = $self->{'f1'};
    
    return if ($calc->indicators->is_available($self->get_name(0), $i));
    return if (! $self->check_dependencies($calc, $i));
    
    my $today = &$getvalue($calc, $i);
    my $yesterday = &$getvalue($calc, $i - 1);

    $calc->indicators->set($self->get_name(0), $i, $today - $yesterday);
    if ($yesterday)
    {
	$calc->indicators->set($self->get_name(1), $i, 
		100 * ($today - $yesterday) / abs($yesterday)
	    );
    } 
}
