package GT::Indicators::MACD;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

# Standards-Version: 1.0

use strict;
use vars qw(@ISA @NAMES @DEFAULT_ARGS);

use GT::Indicators;
use GT::Indicators::EMA;

@ISA = qw(GT::Indicators);
@NAMES = ("MACD[#1,#2,#4]","MACDSignal[#3,#4]","MACDDifference[#1,#2,#3,#4]");
@DEFAULT_ARGS = (12, 26, 9, "{I:Prices CLOSE}");

=pod

=head2 GT::Indicators::MACD

The standard Moving Average Convergence Divergence (MACD 12-26-9) can be called like that : GT::Indicators::MACD->new()

If you need a non standard MACD :
GT::Indicators::MACD->new([20, 50, 15])

=cut

sub initialize {
    my $self = shift;
    
    # We need 3 EMA indicators to calculate the MACD
    $self->{'first_ema'} = GT::Indicators::EMA->new([ $self->{'args'}->get_arg_names(1), $self->{'args'}->get_arg_names(4)]);
    $self->{'second_ema'} = GT::Indicators::EMA->new([ $self->{'args'}->get_arg_names(2), $self->{'args'}->get_arg_names(4)]);
    $self->{'third_ema'} = GT::Indicators::EMA->new([ $self->{'args'}->get_arg_names(3) ],
        $self->get_name(0),
	sub { $_[0]->indicators->get($self->{'first_ema'}->get_name, $_[1]) -
	      $_[0]->indicators->get($self->{'second_ema'}->get_name, $_[1])
	    });

}

=pod

=head2 GT::Indicators::MACD::calculate($calc, $day)

=cut

sub calculate {
    my ($self, $calc, $i) = @_;
    my $indic = $calc->indicators;
    my $macd_name = $self->get_name(0);
    my $signal_name = $self->get_name(1);
    my $diff_name = $self->get_name(2);
    my $nb = $self->{'args'}->get_arg_values($calc, $i, 2);

    return if ( !defined($nb) );

    return if ($indic->is_available($macd_name, $i) &&
	       $indic->is_available($signal_name, $i) &&
	       $indic->is_available($diff_name, $i));

    $self->remove_volatile_dependencies();
    $self->add_volatile_indicator_dependency( $self->{'first_ema'}, $nb);
    $self->add_volatile_indicator_dependency( $self->{'second_ema'}, $nb);
    $self->add_volatile_indicator_dependency( $self->{'third_ema'}, 1);

    return if (! $self->check_dependencies($calc, $i));

    # Get the EMA values and calculate and stores the MACD values
    my $first_ema_value = $indic->get($self->{'first_ema'}->get_name, $i);
    my $second_ema_value = $indic->get($self->{'second_ema'}->get_name, $i);

    $self->{'third_ema'}->calculate($calc, $i);
    my $third_ema_value = $indic->get($self->{'third_ema'}->get_name, $i);

    my $macd = $first_ema_value - $second_ema_value;
    my $signal = $third_ema_value;
    my $diff = $macd - $signal;

    $indic->set($macd_name, $i, $macd);
    $indic->set($signal_name, $i, $signal);
    $indic->set($diff_name, $i, $diff);
}

1;
