package GT::Indicators::VOSC;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Indicators;
use GT::Indicators::SMA;
use GT::Prices;

@ISA = qw(GT::Indicators);
@NAMES = ("VOSC[#1]", "volume");

=pod

=head1 GT::Indicators::VOSC

=head2 Overview

=head2 Calculation

=head2 Examples

GT::Indicators::VOSC->new()
GT::Indicators::VOSC->new([20])

=head2 Links

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my ($args) = @_;
    my $self = { 'args' => defined($args) ? $args : [7] };

    $args->[0] = 7 if (! defined($args->[0]));
    
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my $self = shift;
    
    # Initialize SMA
    $self->{'sma'} = GT::Indicators::SMA->new([ $self->{'args'}[0] ], "SMA", sub { $_[0]->indicators->get($self->get_name(1), $_[1]) });

    # Can't add this dependency since the indicator will not be
    # able to be computed at the check time

    #$self->add_indicator_dependency($self->{'sma'}, 1); 
    $self->add_prices_dependency($self->{'args'}[0]);
}

=pod

=head2 GT::Indicators::VOSC::calculate($calc, $day)

=cut

sub calculate {
    my ($self, $calc, $i) = @_;
    my $indic = $calc->indicators;
    my $prices = $calc->prices;
    my $vosc_name = $self->get_name(0);
    my $volume_name = $self->get_name(1);
    my $volume = 0;

    return if ($indic->is_available($vosc_name, $i));
    return if (! $self->check_dependencies($calc, $i));

    # Determine how much days are required for the calculation
    my $nb_days = $self->{'args'}[0];

    for (my $n = 0; $n < $nb_days; $n++) {

	# Return if volume is available 
	next if $indic->is_available($volume_name, $i - $n);

	# Calculate volume
	if ($prices->at($i - $n)->[$CLOSE] > $prices->at($i - $n)->[$OPEN]) {
	    $volume = $prices->at($i - $n)->[$VOLUME];
	}
	if ($prices->at($i - $n)->[$CLOSE] < $prices->at($i - $n)->[$OPEN]) {
	    $volume = -$prices->at($i - $n)->[$VOLUME];
	}
	if ($prices->at($i - $n)->[$CLOSE] eq $prices->at($i - $n)->[$OPEN]) {
	    $volume = 0;
	}

	# Return results
	$indic->set($volume_name, $i - $n, $volume);

    }

    # Calculate and get VOSC
    $self->{'sma'}->calculate($calc, $i);
    my $vosc_value = $indic->get($self->{'sma'}->get_name, $i);

    # Return the results
    $indic->set($vosc_name, $i, $vosc_value);
}

1;
