package GT::MetaInfo;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;

#use XML::LibXML;

=head1 NAME

GT::MetaInfo - keep various meta informations

=head1 DESCRIPTION

=head2 Goal

This object is used to gather meta informations of different kinds
applying to various objects (state of a trading system, history of order,
support & resistance of prices, top & bottom prices on a period,
lines drawn on a graph, ...). It stores various informations in an
internal XML structure that can be stored in a file and reloaded later.

Informations are stored on a key/value basis. Key is a path in an
XML DOM tree (/ is the separator like for xpath expressions). Several
values can be stored in a single key if you use attributes
to distinguish them.

=head2 API

=over 

=item C<< my $info = GT::MetaInfo->new; >>

Create a new empty GT::MetaInfo object.

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;

    my $self = {};

#    $self->{'_parser'} = XML::LibXML->new();

    return bless $self, $class;
}

=item C<< $info->set($key, $value, { "attrname" => "attrvalue" }); >>

Stores $value corresponding to $key with an attribute "attrname" (whose
value is "attrvalue"). The third parameter is optional if you don't need
attributes for this key.

If the key already existed, then the value is replaced.

=cut
sub set {
    my ($self, $key, $value, $attr) = @_;
    warn "This is not implemented unter Windows\n";
}

=item C<< $info->get($key, { "attrname" => "attrvalue" }); >>

Get the value corresponding the $key and the attributes indicated in the
second optional argument.

=cut
sub get {
    my ($self, $key, $attr) = @_;
    warn "This is not implemented unter Windows\n";
    return;
}

=item C<< my @list = $info->list($key, { "attrname" => "attrvalue" }); >>

List all the set of attributes available for elements corresponding
to the $key. You can eventually restrict the list to a subset og them
by specifying one or more attributes.

Each element of @list is a hash describing the attributes. If you want
the value of that node you have to use $info->get(...).

=cut
sub list {
    my ($self, $key, $attr) = @_;
    warn "This is not implemented unter Windows\n";
}
    
=item C<< $info->load("/path/to/file.xml") >>

Load the XML file as MetaInfo object.

=cut
sub load {
    my ($self, $file) = @_;
    warn "This is not implemented unter Windows\n";
}

=item C<< $info->save("/path/to/file.xml") >

Save the current GT::MetaInfo object in the given XML file. All values
previously set can be reloaded later with $info->load(...).

=cut
sub save {
    my ($self, $file) = @_;
    warn "This is not implemented unter Windows\n";
}

=item C<< $info->dump >>

Output the current internal XML file to the standard output. Useful
for debug purposes.

=cut
sub dump {
    my ($self) = @_;
    warn "This is not implemented unter Windows\n";
}

# Private helper functions
sub _build_xpath_expr {
    my ($key, $attr) = @_;
    warn "This is not implemented unter Windows\n";
}

=pod

=back

=cut
1;
