package GT::OrderFactory::ChannelBreakout;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::OrderFactory;
use Carp::Datum;
use GT::Eval;
use GT::Indicators qw($GET_LAST);
use GT::Tools qw(:generic);

@ISA = qw(GT::OrderFactory);
@NAMES = ("LimitedPrice[#*]");

=head1 GT::OrderFactory::ChannelBreakout

This module is able to set up a generic channel breakout strategy based on
two generic limited price order, above and below current prices. Both
levels will be defined with an indicator.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [] };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    my @function1 = shift_args_until("Indicators:", @args);
    $self->{'f1'} = create_standard_object(@function1);
    $self->{'f1_number'} = extract_object_number(@function1);
    $self->add_indicator_dependency($self->{'f1'}, 1);

    if (scalar @args) {
	my @function2 = shift_args_until("Indicators:", @args);
	$self->{'f2'} = create_standard_object(@function2);
	$self->{'f2_number'} = extract_object_number(@function2);
	$self->add_indicator_dependency($self->{'f2'}, 1);
    } else {
	$self->{'f2'} = $self->{'f1'};
	$self->{'f2_number'} = $self->{'f1_number'};
    }
}

sub create_buy_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;
    my $name = $self->{'f1'}->get_name($self->{'f1_number'});

    return 0 if (! $self->check_dependencies($calc, $i));

    my $price = $calc->indicators->get($name, $i);
    return $pf_manager->buy_conditional($calc, $sys_manager->get_name, $price);
}

sub create_sell_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;
    my $name = $self->{'f2'}->get_name($self->{'f2_number'});

    return 0 if (! $self->check_dependencies($calc, $i));

    my $price = $calc->indicators->get($name, $i);
    return $pf_manager->sell_conditional($calc, $sys_manager->get_name, $price);
}

