package GT::OrderFactory::ClosedToClose;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::OrderFactory;
use Carp::Datum;
use GT::Prices;

@ISA = qw(GT::OrderFactory);
@NAMES = ("ClosedToClose[#1]");

=head1 GT::OrderFactory::StopOnExtreme

Create an order that will be x% above or below current close prices.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 0 ] };
    
    $self->{'args'}[0] = 0 if (! defined($self->{'args'}[0]));

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    $self->{'long_factor'} = 1 + $self->{'args'}[0] / 100;
    $self->{'short_factor'} = 1 - $self->{'args'}[0] / 100;
}

sub create_buy_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;
    
    my $price = $calc->prices->at($i)->[$CLOSE] * $self->{'long_factor'};
    return $pf_manager->buy_limited_price($calc, $sys_manager->get_name, $price);
}

sub create_sell_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;

    my $price = $calc->prices->at($i)->[$CLOSE] * $self->{'short_factor'};
    return $pf_manager->sell_conditional($calc, $sys_manager->get_name, $price);
}

1;
