package GT::OrderFactory::MinimumSlippage;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::OrderFactory;
use Carp::Datum;

@ISA = qw(GT::OrderFactory);
@NAMES = ("MinimumSlippage");

=head1 GT::OrderFactory::MinimumSlippage

=head2 Overview

In a Minimum Slippage test, we rig the software so that buy orders always
catch the minimum possible slippage : all buys occurs at the Low of the
day, and similarly all sells occurs at the High of the day.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [] };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub create_buy_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;

    return $pf_manager->virtual_buy_at_low($calc, $sys_manager->get_name);
}

sub create_sell_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;

    return $pf_manager->virtual_sell_at_high($calc, $sys_manager->get_name);
}

1;
