package GT::OrderFactory::StopOnExtreme;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::OrderFactory;
use Carp::Datum;
use GT::Prices;

@ISA = qw(GT::OrderFactory);
@NAMES = ("StopOnExtreme[#1,#2]");

=head1 GT::OrderFactory::StopOnExtreme

Create a "stop" order that will with the limit the high of the day (modulo
x%) for a long position and the low of the day for a short position.

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 0.5, 1 ] };
    $self->{'args'}[1] = 1 if (! defined($self->{'args'}[1]));

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;
    $self->{'long_factor'} = 1 + $self->{'args'}[0] / 100;
    $self->{'short_factor'} = 1 - $self->{'args'}[0] / 100;

    $self->{'long_second'} = 1 + $self->{'args'}[1] / 100;
    $self->{'short_second'} = 1 - $self->{'args'}[1] / 100;
}

sub create_buy_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;
    
    my $stop = $calc->prices->at($i)->[$HIGH] * $self->{'long_factor'};
    return $pf_manager->buy_conditional($calc, $sys_manager->get_name,
				$stop, $stop * $self->{'long_second'});
}

sub create_sell_order {
    my ($self, $calc, $i, $sys_manager, $pf_manager) = @_;

    my $stop = $calc->prices->at($i)->[$LOW] * $self->{'short_factor'};
    return $pf_manager->sell_conditional($calc, $sys_manager->get_name,
				$stop, $stop * $self->{'short_second'});
}

