package GT::Serializer;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw();

use Carp::Datum;

=head1 NAME

GT::Serializer - Store data in files

=head1 DESCRIPTION

A serializer object is used to store data to files.

=over

=item C<< $self->store_portfolio("file", $portfolio) >>

=item C<< $p = $self->load_portfolio("file") >>

=item C<< $self->store_analysis_list("file", $analysis) >>

=item C<< $a = $self->load_analysis_list("file") >>

=item C<< $self->store_prices("file", $prices) >>

=item C<< $prices = $self->load_prices("file") >>

=cut
sub store_portfolio {
    my ($self, $file, $portfolio) = @_;
    $self->default_store($file, $portfolio);
}
sub load_portfolio {
    my ($self, $file) = @_;
    return $self->default_load($file);
}

sub store_analysis_list {
    my ($self, $file, $analysis) = @_;
    $self->default_store($file, $analysis);
}
sub load_analysis_list { 
    my ($self, $file) = @_;
    return $self->default_load($file);
}

sub store_prices {
    my ($self, $file, $prices);
    $self->default_store($file, $prices);
}
sub load_prices {
    my ($self, $file) = @_;
    return $self->default_load($file);
}

sub new {
    my $type = shift;
    my $class = ref($type) || $type;

    my $self = {};

    return bless $self, $class;
}

=item C<< $self->default_store($file, $object) >>

=item C<< $object = $self->default_load($file) >>

Default generic method able to handle any perl structure.

=cut
sub default_store { 1 }
sub default_load { 1 }

1;
