package GT::Signals::Prices::Advance;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Signals;
use GT::Prices;

@ISA = qw(GT::Signals);
@NAMES = ("Advance[#1]");

=head1 GT::Signals::Advance

The Advance Signal will be able to tell you if a security is advancing more than x % or not from the previous period.
Advance, Decline and Unchange Signals are basics signals and will be your row materials for designing lots of market indicators.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 0 ] };

    $self->{'args'}[0] = 0 if (! defined($self->{'args'}[0]));
    
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;
    
    $self->add_prices_dependency(2);
}

sub detect {
    my ($self, $calc, $i) = @_;
    my $q = $calc->prices;
    my $percentage = ($self->{'args'}[0] / 100);

    return if ($calc->signals->is_available($self->get_name, $i));
    return if (! $self->check_dependencies($calc, $i));
	    
    # We're moving up !
    if ( $q->at($i)->[$LAST] > ($q->at($i-1)->[$LAST] * ( 1 + $percentage))) {
	$calc->signals->set($self->get_name, $i, 1);
    } else {
	$calc->signals->set($self->get_name, $i, 0);
    }
}

1;
