package GT::Signals::Prices::RecordPriceHigh;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Signals;
use GT::Prices;
use GT::Indicators::Generic::MaxInPeriod;

@ISA = qw(GT::Signals);
@NAMES = ("RecordPriceHigh");

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 30 ] };
    
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;
    
    $self->{'max'} = GT::Indicators::Generic::MaxInPeriod->new([ $self->{'args'}[0] - 1 ], "HIGH", $GET_HIGH);

    $self->add_indicator_dependency($self->{'max'}, 2);
    $self->add_prices_dependency($self->{'args'}[0]);
}

sub detect {
    my ($self, $calc, $i) = @_;
    my $q = $calc->prices;
    my $max_name = $self->{'max'}->get_name;

    return if ($calc->signals->is_available($self->get_name, $i));
    return if (! $self->check_dependencies($calc, $i));

    # We're doing a new high
    if ( $q->at($i)->[$HIGH] > $calc->indicators->get($max_name, $i - 1) ) {
        $calc->signals->set($self->get_name, $i, 1);
    } else {
        $calc->signals->set($self->get_name, $i, 0);
    }
}

1;
