package GT::TradeFilters::FollowTrend;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@NAMES @ISA);

use GT::TradeFilters;
use GT::Indicators::SMA;
use Carp::Datum;

@NAMES = ("FollowTrend[#1]");
@ISA = qw(GT::TradeFilters);

=head1 GT::TradeFilters::FollowTrend

This filter  tries to limit the risks by refusing trades againts the
market (ie like buying in a bear market or selling in a bullish market).

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;
    
    my $self = { 'args' => defined($args) ? $args : [50] };
 
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, '');
}

sub initialize {
    my ($self) = @_;
    
    $self->{'mm'} = GT::Indicators::SMA->new([ $self->{'args'}[0] ]);
    
    $self->add_indicator_dependency($self->{'mm'}, 2);
    
    return;
}

sub accept_trade {
    my ($self, $order, $i, $calc, $portfolio) = @_;
    my $mm_name = $self->{'mm'}->get_name;
    
    # Refuse if we can't evaluate the risk
    return 0 if (! $self->check_dependencies_interval($calc, $i - 1, $i));

    if ($calc->indicators->get($mm_name, $i-1) <
	$calc->indicators->get($mm_name, $i))
    {
	# Bull market
	if ($order->{'order'} eq "B")
	{
	    return 1;
	} else {
	    return 0;
	}
    } else {
	# Bear market
	if ($order->{'order'} eq "S")
	{
	    return 1;
	} else {
	    return 0;
	}
    }
}

