package GT::TradeFilters::Generic;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use Carp::Datum;
use GT::Prices;
use GT::TradeFilters;
use GT::Eval;
use GT::Tools qw(:generic);

@ISA = qw(GT::TradeFilters);
@NAMES = ("Generic[#*]");

=head1 TradeFilters::Generic

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => $args };

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    my @args = @{$self->{'args'}};

    my @sig1 = shift_args_until("Signals:", @args);
    
    $self->{'signal'} = create_standard_object(@sig1);
    $self->{'signal_number'} = extract_object_number(@sig1);
    $self->add_signal_dependency($self->{'signal'}, 1);
}

sub precalculate_interval {
    my ($self, $calc, $first, $last) = @_;

    $self->{'signal'}->detect_interval($calc, $first, $last);
}

sub accept_trade {
    my ($self, $order, $i, $calc, $portfolio) = @_;

    return if (! $self->check_dependencies($calc, $i));

    if ($calc->signals->get($self->{'signal'}->get_name($self->{'signal_number'}), $i)) {
        return 1;
    } else {
        return 0;
    }
}

1;
