package GT::TradeFilters::OneTrade;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@NAMES @ISA);

use GT::TradeFilters;
use Carp::Datum;

@NAMES = ("SingleTrade");
@ISA = qw(GT::TradeFilters);

=head1 GT::TradeFilters::OneTrade

This filter refuses a new trade if a position is actually open. It will
however accept it if it's in the process of being closed.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;
    
    my $self = { 'args' => defined($args) ? $args : [50] };
 
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, '');
}

sub accept_trade {
    my ($self, $order, $i, $calc, $portfolio) = @_;
    
    my @pos = $portfolio->list_open_positions($order->source);
    foreach (@pos)
    {
	if ($_->code eq $order->code)
	{
	    if (! $_->being_closed)
	    {
		return 0;
	    }
	}
    }
    return 1;
}

